@@ -25,23 +25,41 @@
 */
 
 #import <Foundation/Foundation.h>
+
 #import "fileElement.h"
 
-
-@interface client : NSObject
+/**
+ * Clas that represents an object to access files, be them remote or local.
+ * It must be subclassed to be useful. This object holds porperties genrealy
+ * valid for both local and remote instances as the working directory and the
+ * home directory of the user. It also defines common methods for creating,
+ * changing and deleting directories.
+ */
+@interface Client : NSObject
 {
+    id       controller;
     NSString *workingDir;
     NSString *homeDir;
 }
 
+- (id)init;
+- (id)initWithController:(id)cont;
+
+/** returns the current working directory */
 - (NSString *)workingDir;
+
 - (void)setWorkingDirWithCString:(char *)dir;
 - (void)setWorkingDir:(NSString *)dir;
 - (void)changeWorkingDir:(NSString *)dir;
 - (BOOL)createNewDir:(NSString *)dir;
-- (void)deleteFile:(fileElement *)file beingAt:(int)depth;
+- (void)deleteFile:(FileElement *)file beingAt:(int)depth;
+
 - (NSArray *)workDirSplit;
+
+/** returns an array with the directory listing */
 - (NSArray *)dirContents;
+
+/** returns the current home directory */
 - (NSString *)homeDir;
 @end
 
