@@ -1,7 +1,7 @@
 /*
  Project: FTP
 
- Copyright (C) 2005 Riccardo Mottola
+ Copyright (C) 2005-2010 Riccardo Mottola
 
  Author: Riccardo Mottola
 
@@ -30,19 +30,27 @@
  * the connection modes, default, active (port) and passive
  * can be set using the three setPort* methods
  */
- 
+
+
 #import "ftpclient.h"
 #import "AppController.h"
 #import "fileElement.h"
 
-#include <arpa/inet.h>  /* for inet_ntoa and similar */
-#include <netdb.h>
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <time.h>
 
+#ifdef _WIN32
+#include <fcntl.h>
+#else
+#include <arpa/inet.h>  /* for inet_ntoa and similar */
+#include <netdb.h>
+#define INVALID_SOCKET -1
+#define closesocket close
+#endif /* WIN32 */
+
 
 #define MAX_CONTROL_BUFF 2048
 #define MAX_DATA_BUFF 2048
@@ -53,24 +61,83 @@
 #define socklentype int
 #endif
 
-
-@implementation ftpclient
-
-/* initializer */
-/* we set possibly unused stuff to NULL */
-- (id)init
+void initStream(streamStruct *ss, int socket)
 {
-    if (!(self =[super init]))
-        return nil;
-    controller = nil;
-    return self;
+    ss->socket = socket;
+    ss->position = 0;
+    ss->len = 0;
+    ss->buffer[0] = '\0';
+}
+
+int getChar(streamStruct* ss)
+{
+    int result;
+    BOOL gotEof;
+
+    gotEof = NO;
+    if (ss->position == ss->len)
+    {
+        int read;
+
+        read = recv(ss->socket, ss->buffer, MAX_SOCK_BUFF, 0);
+        if (read > 0)
+        {
+            ss->len = read;
+            ss->position = 0;
+        } else if (read == 0)
+        {
+            ss->len = 0;
+            ss->position = 0;
+            ss->buffer[0] = '\0';
+            gotEof = YES;
+        } else
+        {
+            ss->len = 0;
+            ss->position = 0;
+            NSLog(@"error sock read");
+            perror("getChar:read");
+            ss->buffer[0] = '\0';
+            gotEof = YES;
+        }
+    }
+    if (gotEof)
+        result = EOF;
+    else {    
+        result = ss->buffer[ss->position];
+        ss->position++;
+    }
+    return result;
+}
+
+@implementation FtpClient
+
++ (void)connectWithPorts:(NSArray *)portArray
+{
+    NSAutoreleasePool *pool;
+    NSConnection *serverConnection;
+    FtpClient    *serverObject;
+	
+    pool = [[NSAutoreleasePool alloc] init];
+	
+    serverConnection = [NSConnection
+            connectionWithReceivePort:[portArray objectAtIndex:0]
+							 sendPort:[portArray objectAtIndex:1]];
+	
+    serverObject = [self alloc];
+    [(id)[serverConnection rootProxy] setServer:serverObject];
+    [serverObject release];
+	
+    [[NSRunLoop currentRunLoop] run];
+    [pool release];
+	
+    return;
 }
 
 - (id)initWithController:(id)cont :(connectionModes)cMode
 {
-    if (!(self =[super init]))
+    if (!(self =[super initWithController:cont]))
         return nil;
-    controller = cont;
+
     switch (cMode)
     {
         case defaultMode:
@@ -85,6 +152,15 @@
         default:
             [self setPortDefault];
     }
+#ifdef _WIN32
+    WORD wVersionRequested;
+    WSADATA wsaData;
+    wVersionRequested = MAKEWORD( 1, 1 );
+
+    WSAStartup(wVersionRequested, &wsaData);
+    NSLog(@"inited WinSock");
+#endif
+    connected = NO;
     return self;
 }
 
@@ -117,6 +193,9 @@
     char            tempStr2[MAX_CONTROL_BUFF];
     NSMutableArray *reply;
 
+    if (!connected)
+        return;
+	
     [dir getCString:tempStr2];
     sprintf(tempStr, "CWD %s\r\n", tempStr2);
     [self writeLine:tempStr];
@@ -146,6 +225,8 @@
  returned is the first numerical code
  NOTE: the parser is NOT robust in handling errors
  */
+#define NUMCODELEN 4
+
 - (int)readReply :(NSMutableArray **)result
 {
     char  buff[MAX_CONTROL_BUFF];
@@ -153,7 +234,7 @@
     int   ch;
     /* the first numerical code, in case of multi-line output it is followed
        by '-' in the first line and by ' ' in the last line */
-    char  numCodeStr[4];
+    char  numCodeStr[NUMCODELEN];
     int   numCode;
     int   startNumCode;
     char  separator;
@@ -165,17 +246,23 @@
     state = N1;
     separator = 0;
     multiline = NO;
+    startNumCode = numCode = 0;
     *result = [NSMutableArray arrayWithCapacity:1];
 
-
+    // TODO: protect against numCodeStr overflow
     while (!(state == END))
     {
-        ch = getc(controlInStream);
+        ch = getChar(&ctrlStream);
+//	NSLog(@"read char: %c", ch);
+        if (ch == EOF)
+            state = END;
+
         switch (state)
         {
             case N1:
                 buff[readBytes] = ch;
-                numCodeStr[readBytes] = ch;
+                if (readBytes < NUMCODELEN)
+                    numCodeStr[readBytes] = ch;
                 readBytes++;
                 if (ch == ' ') /* skip internal lines of multi-line */
                     state = CHARS;
@@ -236,6 +323,10 @@
                     }
                 }
                 break;
+            case END:
+                NSLog(@"EOF reached prematurely");
+		startNumCode = -1;
+                break;
             default:
                 NSLog(@"Duh, a case default in the readReply parser");
         }
@@ -245,41 +336,62 @@
 }
 
 /*
- writes a single line to the control connection
+ writes a single line to the control connection, logging it always
  */
 - (int)writeLine:(char *)line
+{
+    return [self writeLine:line byLoggingIt:YES];
+}
+
+/*
+ writes a single line to the control connection
+ */
+- (int)writeLine:(char *)line byLoggingIt:(BOOL)doLog
 {
     int sentBytes;
     int bytesToSend;
-    
+
     bytesToSend = strlen(line);
-    [self logIt:[NSString stringWithCString:line length:(bytesToSend - 2)]];
+    if (doLog)
+        [self logIt:[NSString stringWithCString:line length:(bytesToSend - 2)]];
     if ((sentBytes = send(controlSocket, line, strlen(line), 0)) < bytesToSend)
         NSLog(@"sent %d out of %d", sentBytes, bytesToSend);
     return sentBytes;
 }
 
+
 - (int)setTypeToI
 {
     NSMutableArray *reply;
+    int            retVal;
     
-    [self writeLine:"TYPE I\r\n"];
-    [self readReply:&reply];
-    [reply release];
-    return 0;
+    retVal = [self writeLine:"TYPE I\r\n"];
+    if ( retVal > 0)
+    {
+        [self readReply:&reply];
+        [reply release];
+    }
+       
+    return retVal;
 }
 
 - (int)setTypeToA
 {
     NSMutableArray *reply;
+    int            retVal;
 
-    [self writeLine:"TYPE A\r\n"];
-    [self readReply:&reply];
-    [reply release];
-    return 0;
+    retVal = [self writeLine:"TYPE A\r\n"];
+    NSLog(@"retval: %d", retVal);
+    if ( retVal > 0)
+    {
+        [self readReply:&reply];
+        [reply release];
+    }
+    
+    return retVal;
 }
 
-- (void)retrieveFile:(fileElement *)file to:(localclient *)localClient beingAt:(int)depth
+- (oneway void)retrieveFile:(FileElement *)file to:(LocalClient *)localClient beingAt:(int)depth;
 {
     NSString           *fileName;
     unsigned long long fileSize;
@@ -291,16 +403,19 @@
     NSMutableArray     *reply;
     struct sockaddr    from;
     int                fromLen;
+    unsigned int       minimumPercentIncrement;
+    unsigned int       progressIncBytes;
     int                replyCode;
     unsigned long long totalBytes;
     NSString           *localPath;
     BOOL               gotFile;
 
-    fromLen = sizeof(from);
+    fromLen = sizeof(from);    
 
-    NSLog(@"filesize should be %u", (unsigned)[file size]);
     fileName = [file filename];
     fileSize = [file size];
+    minimumPercentIncrement = fileSize / 100; // we should guard against maxint
+
     localPath = [[localClient workingDir] stringByAppendingPathComponent:fileName];
 
     if ([file isDir])
@@ -310,9 +425,9 @@
         NSArray      *dirList;
         NSString     *remoteDir;
         NSEnumerator *en;
-        fileElement  *fEl;
+        FileElement  *fEl;
 
-        if (depth > 5)
+        if (depth > MAX_DIR_RECURSION)
         {
             NSLog(@"Max depth reached: %d", depth);
             return;
@@ -330,10 +445,10 @@
     
             dirList = [self dirContents];
             en = [dirList objectEnumerator];
-            while (fEl = [en nextObject])
+            while ((fEl = [en nextObject]))
             {
                 NSLog(@"recurse, download : %@", [fEl filename]);
-                [self retrieveFile:fEl to:localClient beingAt:(depth+1)];
+                [self retrieveFile:fEl to:localClient beingAt:depth+1];
             }
         }
         /* we get back were we started */
@@ -360,25 +475,35 @@
     NSLog(@"%d reply is %@: ", replyCode, [reply objectAtIndex:0]);
 
     if(replyCode != 150)
+    {
+        [controller showAlertDialog:@"Unexpected server error."];
+        NSLog(@"Unexpected condition in retrieve");
         return; /* we have an error or some unexpected condition */
+    }
     [reply release];
     
     if ([self initDataStream] < 0)
+    {
+        [controller showAlertDialog:@"Unexpected connection error."];
         return;
+    }
     
     localFileStream = fopen([localPath cString], "w");
     if (localFileStream == NULL)
     {
+        [controller showAlertDialog:@"Opening of local file failed.\nCheck permissions and free space."];
         perror("local fopen failed");
         return;
     }
     
     totalBytes = 0;
+    progressIncBytes = 0;
     gotFile = NO;
+    [controller setThreadRunningState:YES];
     [controller setTransferBegin:fileName :fileSize];
     while (!gotFile)
     {
-        bytesRead = read(localSocket, buff, MAX_DATA_BUFF);
+        bytesRead = recv(localSocket, buff, MAX_DATA_BUFF, 0);
         if (bytesRead == 0)
             gotFile = YES;
         else if (bytesRead < 0)
@@ -392,19 +517,25 @@
                 NSLog(@"file write error, retrieve file");
             }
             totalBytes += bytesRead;
-            [controller setTransferProgress:totalBytes];
+            progressIncBytes += bytesRead;
+            if (progressIncBytes > minimumPercentIncrement) 
+            {
+                [controller setTransferProgress:[NSNumber numberWithUnsignedLongLong:totalBytes]];
+                progressIncBytes = 0;
+            }
         }
     }
-    [controller setTransferEnd:totalBytes];
+
+    [controller setTransferEnd:[NSNumber numberWithUnsignedLongLong:totalBytes]];
     
-    NSLog(@"transferred %u", (unsigned long)totalBytes);
     fclose(localFileStream);
     [self closeDataStream];
     [self readReply:&reply];
     [reply release];
+    [controller setThreadRunningState:NO];
 }
 
-- (void)storeFile:(fileElement *)file from:(localclient *)localClient beingAt:(int)depth
+- (oneway void)storeFile:(FileElement *)file from:(LocalClient *)localClient beingAt:(int)depth
 {
     NSString           *fileName;
     unsigned long long fileSize;
@@ -416,16 +547,20 @@
     int                bytesRead;
     struct sockaddr    from;
     int                fromLen;
+    unsigned int       minimumPercentIncrement;
+    unsigned int       progressIncBytes;
     int                replyCode;
-    unsigned           totalBytes;
+    unsigned long long totalBytes;
     NSString           *localPath;
     BOOL               gotFile;
 
+    
     fromLen = sizeof(from);
 
     fileName = [file filename];
     fileSize = [file size];
-    
+    minimumPercentIncrement = fileSize / 100; // we should guard against maxint
+
     localPath = [[localClient workingDir] stringByAppendingPathComponent:fileName];
 
     if ([file isDir])
@@ -435,9 +570,9 @@
         NSArray      *dirList;
         NSString     *remotePath;
         NSEnumerator *en;
-        fileElement  *fEl;
+        FileElement  *fEl;
 
-        if (depth > 3)
+        if (depth > MAX_DIR_RECURSION)
         {
             NSLog(@"Max depth reached: %d", depth);
             return;
@@ -453,12 +588,12 @@
 
         if ([self createNewDir:remotePath] == YES)
         {
-            NSLog(@"remote dir created succesfully");
+            NSLog(@"remote dir created successfully");
             [self changeWorkingDir:remotePath];
 
             dirList = [localClient dirContents];
             en = [dirList objectEnumerator];
-            while (fEl = [en nextObject])
+            while ((fEl = [en nextObject]))
             {
                 NSLog(@"recurse, upload : %@", [fEl filename]);
                 [self storeFile:fEl from:localClient beingAt:(depth+1)];
@@ -477,7 +612,8 @@
 
     if ([self initDataConn] < 0)
     {
-        NSLog(@"error initiating data connection, retrieveFile");
+        [controller showAlertDialog:@"Error initiating the Data Connection."];
+        NSLog(@"error initiating data connection, storeFile");
         return;
     }
 
@@ -486,21 +622,36 @@
     [self writeLine:command];
     replyCode = [self readReply:&reply];
     NSLog(@"%d reply is %@: ", replyCode, [reply objectAtIndex:0]);
+
+    if (replyCode >= 550 && replyCode <= 559)
+    {
+        [controller showAlertDialog:[reply objectAtIndex:0]];
+        [self logIt: [reply objectAtIndex:0]];
+        [reply release];
+        return;
+    }
     [reply release];
 
+    
     if ([self initDataStream] < 0)
+    {
+        [controller showAlertDialog:@"Unexpected connection error."];
         return;
+    }
 
 
     localFileStream = fopen([localPath cString], "r");
     if (localFileStream == NULL)
     {
+        [controller showAlertDialog:@"Opening of local file failed.\n Check permissions."];
         perror("local fopen failed");
         return;
     }
 
     totalBytes = 0;
+    progressIncBytes = 0;
     gotFile = NO;
+    [controller setThreadRunningState:YES];
     [controller setTransferBegin:fileName :fileSize];
     while (!gotFile)
     {
@@ -519,19 +670,24 @@
                 NSLog(@"socket write error, store file");
             }
             totalBytes += bytesRead;
-            [controller setTransferProgress:totalBytes];
+            progressIncBytes += bytesRead;
+            if (progressIncBytes > minimumPercentIncrement) 
+            {
+                [controller setTransferProgress:[NSNumber numberWithUnsignedLongLong:totalBytes]];
+                progressIncBytes = 0;
+            }
         }
     }
-    [controller setTransferEnd:totalBytes];
+    [controller setTransferEnd:[NSNumber numberWithUnsignedLongLong:totalBytes]];
     
-    NSLog(@"transferred %u", totalBytes);
     fclose(localFileStream);
     [self closeDataStream];
     [self readReply:&reply];
     [reply release];
+    [controller setThreadRunningState:NO];
 }
 
-- (void)deleteFile:(fileElement *)file beingAt:(int)depth
+- (void)deleteFile:(FileElement *)file beingAt:(int)depth
 {
     NSString           *fileName;
     NSString           *localPath;
@@ -550,7 +706,7 @@
         NSArray      *dirList;
         NSString     *remotePath;
         NSEnumerator *en;
-        fileElement  *fEl;
+        FileElement  *fEl;
 
         if (depth > 3)
         {
@@ -563,12 +719,12 @@
         NSLog(@"it is a dir: %@", fileName);
         remotePath = [pristineRemotePath stringByAppendingPathComponent:fileName];
 
-        NSLog(@"remote dir created succesfully");
+        NSLog(@"remote dir created successfully");
         [self changeWorkingDir:remotePath];
 
         dirList = [self dirContents];
         en = [dirList objectEnumerator];
-        while (fEl = [en nextObject])
+        while ((fEl = [en nextObject]))
         {
             NSLog(@"recurse, delete : %@", [fEl filename]);
             [self deleteFile:fEl beingAt:(depth+1)];
@@ -603,13 +759,13 @@
         NSLog(@"Could not resolve %s", server);
         return ERR_COULDNT_RESOLVE;
     }
-    bcopy((char *)hostentPtr->h_addr, (char *)&remoteSockName.sin_addr, hostentPtr->h_length);
+    BCOPY((char *)hostentPtr->h_addr, (char *)&remoteSockName.sin_addr, hostentPtr->h_length);
     remoteSockName.sin_family = PF_INET;
     remoteSockName.sin_port = htons(port);
 
     tempStr = inet_ntoa(remoteSockName.sin_addr);
 
-    if ((controlSocket = socket(PF_INET, SOCK_STREAM, 0)) < 0)
+    if ((controlSocket = socket(PF_INET, SOCK_STREAM, 0)) == INVALID_SOCKET)
     {
         perror("socket failed: ");
         return ERR_SOCKET_FAIL;
@@ -629,8 +785,9 @@
         return ERR_GESOCKNAME_FAIL;
     }
     
-    controlInStream = fdopen(controlSocket, "r");
-    [self readReply :&reply];
+    initStream(&ctrlStream, controlSocket);
+    if([self readReply :&reply] < 0)
+      return ERR_READ_FAIL;
     [reply release];
     return 0;
 }
@@ -641,6 +798,7 @@
     
     [self writeLine:"QUIT\r\n"];
     [self readReply:&reply];
+    connected = NO;
 }
 
 - (int)authenticate:(char *)user :(char *)pass
@@ -650,9 +808,9 @@
     int            replyCode;
 
     sprintf(tempStr, "USER %s\r\n", user);
-    [self writeLine:tempStr];
+    if ([self writeLine:tempStr] < 0)
+      return -2; /* we couldn't write */
     replyCode = [self readReply:&reply];
-    NSLog(@"user reply is: %@", [reply objectAtIndex:0]);
     if (replyCode == 530)
     {
         NSLog(@"Not logged in: %@", [reply objectAtIndex:0]);
@@ -663,9 +821,9 @@
     [reply release];
     
     sprintf(tempStr, "PASS %s\r\n", pass);
-    [self writeLine:tempStr];
+    if ([self writeLine:tempStr byLoggingIt:NO] < 0)
+      return -2; /* we couldn't write */
     replyCode = [self readReply:&reply];
-    NSLog(@"pass reply is: %@", [reply objectAtIndex:0]);
     if (replyCode == 530)
     {
         NSLog(@"Not logged in: %@", [reply objectAtIndex:0]);
@@ -674,6 +832,8 @@
         return -1;
     }
     [reply release];
+    
+    connected = YES;
 
     /* get home directory as dir we first connected to */
     [self writeLine:"PWD\r\n"];
@@ -725,7 +885,7 @@
         int            a1, a2, a3, a4;
         int            p1, p2;
         
-        if ((dataSocket = socket(AF_INET, SOCK_STREAM, 0)) < 0)
+        if ((dataSocket = socket(AF_INET, SOCK_STREAM, 0)) == INVALID_SOCKET)
         {
             perror("socket in initDataConn");
             return -1;
@@ -802,7 +962,7 @@
     if (usesPorts == YES)
         dataSockName.sin_port = 0;
     
-    if ((dataSocket = socket(AF_INET, SOCK_STREAM, 0)) < 0)
+    if ((dataSocket = socket(AF_INET, SOCK_STREAM, 0)) == INVALID_SOCKET)
     {
         perror("socket in initDataConn");
         return -1;
@@ -881,7 +1041,7 @@
     fromLen = sizeof(from);
     if (usesPassive)
     {
-        dataStream = fdopen(dataSocket, "r");
+        initStream(&dataStream, dataSocket);
         localSocket = dataSocket;
     } else
     {
@@ -889,33 +1049,32 @@
         {
             perror("accepting socket, initDataStream: ");
         }
-        dataStream = fdopen(localSocket, "r");
+        initStream(&dataStream, localSocket);
     }
-
+/*
     if (dataStream == NULL)
     {
         perror("data stream opening failed");
         return -1;
-    }
+    } */
     NSLog(@"data stream open");
     return 0;
 }
 
 - (int)closeDataConn
 {
-    close (dataSocket);
+    closesocket(dataSocket);
     return 0;
 }
 
 - (void)closeDataStream
 {
-    fclose (dataStream);
     // a passive localSocket is just a copy of the dataSocket
     if (usesPassive == NO)
-        close(localSocket);
+        closesocket(localSocket);
     // apparently it is not true that fclose closes the underlying
     // descriptor, without closing dataSocket we got a bind error
-    // at the next conneciton attempt
+    // at the next connection attempt
     [self closeDataConn];
 }
 
@@ -965,14 +1124,23 @@
     enum               states_m1 { READ, GOTR };
     enum               states_m1 state;
     NSMutableArray     *listArr;
-    fileElement        *aFile;
+    FileElement        *aFile;
     char               path[4096];
     NSMutableArray     *reply;
-
+    int                replyCode;
+    
+    if (!connected)
+        return nil;
+    
     [workingDir getCString:path];
 
     /* lets settle to a plain ascii standard type */
-    [self setTypeToA];
+    if ([self setTypeToA] < 0)
+    {
+        connected = NO;
+        NSLog(@"Timed out.");
+        return nil;
+    }
     
     /* create an array with a reasonable starting size */
     listArr = [NSMutableArray arrayWithCapacity:5];
@@ -983,11 +1151,11 @@
 
     if ([self initDataStream] < 0)
         return nil;
-    
+
     /* read the directory listing, each line being CR-LF terminated */
     state = READ;
     readBytes = 0;
-    while ((ch = getc(dataStream)) != EOF)
+    while ((ch = getChar(&dataStream)) != EOF)
     {
         if (ch == '\r')
             state = GOTR;
@@ -996,23 +1164,26 @@
             
             buff[readBytes] = '\0';
             fprintf(stderr, "%s\n", buff);
+            [self logIt:[NSString stringWithCString:buff]];
             state = READ; /* reset the state for a new line */
             readBytes = 0;
-            aFile = [[fileElement alloc] initWithLsLine:buff];
+            aFile = [[FileElement alloc] initWithLsLine:buff];
             if (aFile)
                 [listArr addObject:aFile];
         } else
             buff[readBytes++] = ch;
     }
-    if (ferror(dataStream))
+/* FIXME ***********    if (ferror(dataStream))
     {
         perror("error in reading data stream: ");
     } else if (feof(dataStream))
     {
          fprintf(stderr, "feof\n");
-    }
+    } */
     [self closeDataStream];
-    [self readReply:&reply];
+
+    replyCode = [self readReply:&reply];
+    
     return [NSArray arrayWithArray:listArr];
 }
 
